/*  Dynkin project programs
 *  Copyright (C) 1998 Anders S. Buch (abuch@mi.aau.dk)
 *  See the file LICENSE for license information.
 */

/*  Dynkin project programs
 *  Copyright (C) 1998 Anders S. Buch (abuch@mi.aau.dk)
 *  See the file LICENSE for license information.
 */

#include <ctype.h>
#include <unistd.h>
#include <stdlib.h>
#include <stdarg.h>

#include "alloc.h"
#include "matrix.h"
#include "list.h"
#include "calc.h"
#include "cmdline.h"


int ac;
char **av;

void cmd_init(int _ac, char **_av)
{
  ac = _ac;
  av = _av;
}

void cmd_error(char *fmt, ...)
{
  va_list ap;
  va_start(ap, fmt);
  fputs(av[0], stderr);
  fputs(": ", stderr);
  vfprintf(stderr, fmt, ap);
  va_end(ap);
  exit(1);
}


affine *cmd_rsys()
{
  affine *afn = NULL;
  char *rs = NULL, *env = NULL;
  
  if (optind == 0)
    optind++;
  
  if (optind < ac && (afn = init_affine(rs = av[optind])) != NULL)
    {
      optind++;
      return afn;
    }
  
  env = getenv("ROOTSYSTEM");
  if (env != NULL && (afn = init_affine(env)) != NULL)
    return afn;
  
  if (env != NULL)
    rs = env;
  if (rs != NULL)
    fprintf(stderr, "%s: Unknown root system `%s'\n", av[0], rs);
  print_usage(av[0]);
  
  return NULL;
}


list *cmd_wge(affine *afn)
{
  list *wge = l_new();
  
  if (optind == 0)
    optind++;
  
  while (optind < ac)
    {
      char *s = av[optind];
      
      if (*s == '{' && l_length(wge) == 0)
	s++;
      
      if (isdigit(*s))
	{
	  l_appendint(wge, atoi(s));
	  optind++;
	}
      else if (strcmp(s, "long") == 0)
	{
	  int i;
	  for (i = 0; i < l_length(afn->longest_element); i++)
	    l_appendint(wge, l_intelem(afn->longest_element, i));
	  optind++;
	}
      else if (strcmp(s, "top") == 0)
	{
	  int i;
	  for (i = 0; i < l_length(afn->top_alcove); i++)
	    l_appendint(wge, l_intelem(afn->top_alcove, i));
	  optind++;
	}
      else
	break;
    }
  
  reduced_exp(wge, wge, afn);
  
  return wge;
}


vector *cmd_weight(int add, affine *afn)
{
  vector *w;
  int i, n = afn_dim(afn);
  
  if (optind == 0)
    optind++;
  
  if (ac - optind < n)
    print_usage(av[0]);
  
  w = v_new(n);
  for (i = 0; i < n; i++)
    {
      char *s = av[optind++];
      char *t;
      
      if (*s == '(' && i == 0)
	s++;
      
      t = s;
      if (*t == '-')
	t++;
      if (! isdigit(*t))
	print_usage(av[0]);
      
      v_entry(w, i) = atoi(s) + add;
    }
  
  return w;
}


int cmd_dash(int insist)
{
  char *s;
  
  if (optind == 0)
    optind++;
  
  s = av[optind];
  
  if (s != NULL && s[0] == '-' && s[1] == '\0')
    {
      optind++;
      return 1;
    }
  
  if (insist)
    print_usage(av[0]);
  
  return 0;
}


char *cmd_file(int insist)
{
  if (optind == 0)
    optind++;
  if (optind < ac)
    return av[optind++];
  if (insist)
    print_usage(av[0]);
  return NULL;
}
