/*  Dynkin project programs
 *  Copyright (C) 1998 Anders S. Buch (abuch@mi.aau.dk)
 *  See the file LICENSE for license information.
 */

#include "efhtree.h"
#include "alloc.h"

efh_tree *efh_new(list *efh)
{
  int ln = l_length(efh);
  int i;
  efh_tree *t = (efh_tree *) 
    weyl_calloc(1, sizeof(efh_tree) + (ln - 1) * sizeof(char));
  efh_length(t) = ln;
  for (i = 0; i < ln; i++)
    efh_elem(t, i) = l_intelem(efh, i);
  return t;
}

void efh_free(efh_tree *t)
{
  if (t == NULL)
    return;
  efh_free(efh_left(t));
  efh_free(efh_right(t));
  weyl_free(t);
}

int efh_comp(efh_tree *t, list *efh)
{
  int d, i, ln = l_length(efh);
  d = efh_length(t) - ln;
  if (d) return d;
  for (i = 0; i < ln; i++)
    {
      d = efh_elem(t, i) - l_intelem(efh, i);
      if (d) return d;
    }
  return 0;
}

int efh_insert(efh_tree *t, list *efh, efh_tree **node)
{
  efh_tree *last = NULL;
  int c = 1;
  while (c != 0 && t != NULL)
    {
      last = t;
      c = efh_comp(t, efh);
      if (c < 0)
	t = efh_left(t);
      else if (c > 0)
	t = efh_right(t);
    }
  
  if (c == 0)
    {
      *node = t;
      return 1;
    }
  
  *node = efh_new(efh);
  if (c < 0)
    efh_left(last) = *node;
  else
    efh_right(last) = *node;
  return 0;
}
