/*  Dynkin project programs
 *  Copyright (C) 1998 Anders S. Buch (abuch@mi.aau.dk)
 *  See the file LICENSE for license information.
 */

#include <string.h>

#include <stdio.h>
#include <ctype.h>

#include "figlib.h"

int fig_xsize;
int fig_ysize;

FILE *fig_file;

char *pap_name[] = {"Letter", "Legal", "Ledger", "A4", "B5", NULL};
int pap_xsz[] = {85, 110, 110, 83, 72};
int pap_ysz[] = {110, 140, 170, 118, 102};

int get_sz10(char **s)
{
  int res = 0;
  while (isdigit(**s))
    {
      res = 10 * res + **s - '0';
      (*s)++;
    }
  res *= 10;
  if (**s == '.')
    {
      (*s)++;
      if (isdigit(**s))
	res += **s - '0';
      while (isdigit(**s))
	(*s)++;
    }
  return res;
}

int fig_set_page(char *page_size)
{
  int i, xsz, ysz;
  char *sz;
  
  if (page_size == NULL)
    {
      fig_xsize = FIG_RESOLUTION * pap_xsz[0] / 10;
      fig_ysize = FIG_RESOLUTION * pap_ysz[0] / 10;
      return 0;
    }
  
  for (i = 0; pap_name[i] != NULL; i++)
    if (strcmp(page_size, pap_name[i]) == 0)
      break;
  if (pap_name[i] != NULL)
    {
      fig_xsize = FIG_RESOLUTION * pap_xsz[i] / 10;
      fig_ysize = FIG_RESOLUTION * pap_ysz[i] / 10;
      return 0;
    }
  
  sz = page_size;
  xsz = get_sz10(&sz);
  if (*sz == 'x')
    sz++;
  ysz = get_sz10(&sz);
  
  if (ysz > 0)
    {
      fig_xsize = FIG_RESOLUTION * xsz / 10;
      fig_ysize = FIG_RESOLUTION * ysz / 10;
      return 0;
    }
  
  return -1;
}

void fig_init(FILE *s)
{
  fig_file = (s != NULL) ? s : stdout;
  fputs("#FIG 3.1\nPortrait\nCenter\nMetric\n1200 2\n", fig_file);
}

void fig_end()
{
  fflush(fig_file);
}

void fig_line(int x1, int y1, int x2, int y2)
{
  fprintf(fig_file, "2 1 0 1 -1 7 0 0 -1 0.000 0 0 -1 0 0 2\n"
	  "\t%d %d %d %d\n", x1, y1, x2, y2);
}

void fig_text(int x, int y, char *s, int center)
{
  int len = strlen(s);
  if (center)
    {
      x -= len * FIG_CHAR_XSZ / 2;
      y += FIG_CHAR_YSZ / 2;
    }
  fprintf(fig_file, "4 0 0 0 0 0 12 0.0000 4 135 %d %d %d %s\\001\n",
	  FIG_CHAR_XSZ * len, x, y, s);
}
