/*  Dynkin project programs
 *  Copyright (C) 1998 Anders S. Buch (abuch@mi.aau.dk)
 *  See the file LICENSE for license information.
 */

#include "alloc.h"
#include "output.h"
#include "affine.h"
#include "matrix.h"
#include "weight.h"
#include "freudlib.h"
#include "cmdline.h"


void print_usage(char *s)
{
  fprintf(stderr, "Usage: %s rsys weight\n", s);
  exit(1);
}

int main(int ac, char **av)
{
  int n, i, j;
  vector *a, *d, *high_weight;
  list *omega, *sums;
  int *action_tab;
  weight_set *hw_set;
  affine *afn;
  
  cmd_init(ac, av);
  afn = cmd_rsys();
  high_weight = cmd_weight(0, afn);
  
  n = afn_dim(afn);
  a = v_new(n);
  d = v_new(n);
  
  action_tab = build_action_tab(afn);
  hw_set = freudenthal(high_weight, afn, action_tab);
  weyl_free(action_tab);
  
  omega = ws_to_list(hw_set);
  
  for (i = 0; i < l_length(omega); i++)
    {
      weight_set *ws = l_element(omega, i);
      vector *w = ws_weight(ws);
      
      v_sub(d, high_weight, w);
      m_multvec(a, afn->atcartan, d);
      v_div(a, a, afn->dcartan);
      
      sums = generate_sums(a, afn);
      
      for (j = 0; j < n; j++)
	printf("%2d ", v_entry(w, j));
      
      printf(" :  m = %d ; p = %d", ws_mult(ws), l_length(sums));
      if (l_length(sums) == ws_mult(ws))
	putchar('\n');
      else if (l_length(sums) > ws_mult(ws))
	printf("         m < p\n");
      else
	printf("         m > p\n");
      
      free_sums(sums);
    }
  
  afn_free(afn);
  v_free(high_weight);
  v_free(a);
  v_free(d);
  l_free(omega);
  ws_free(hw_set);
  
  memory_report;
  
  exit(0);
}
