/*  Dynkin project programs
 *  Copyright (C) 1998 Anders S. Buch (abuch@mi.aau.dk)
 *  See the file LICENSE for license information.
 */

#include <stdio.h>
#include <unistd.h>

#include "alloc.h"
#include "matrix.h"
#include "affine.h"
#include "output.h"
#include "cmdline.h"


void print_usage(char *s)
{
  fprintf(stderr, "Usage: %s [-c] rsys weight\n", s);
  exit(1);
}

int main(int ac, char **av)
{
  affine *afn;
  vector *a;
  list *sums;
  int c, opt_count;
  
  cmd_init(ac, av);
  
  c = getopt(ac, av, "c");
  if (c == '?')
    print_usage(av[0]);
  opt_count = (c == 'c');
  
  afn = cmd_rsys();
  a = cmd_weight(0, afn);
  
  sums = generate_sums(a, afn);
  
  if (opt_count)
    printf("%d\n", l_length(sums));
  else
    print_vec_list(sums);
  
  afn_free(afn);
  v_free(a);
  free_sums(sums);
  memory_report;
  
  exit(0);
}
