/*  Dynkin project programs
 *  Copyright (C) 1998 Anders S. Buch (abuch@mi.aau.dk)
 *  See the file LICENSE for license information.
 */

#include <stdio.h>
#include <stdlib.h>

#include "list.h"
#include "matrix.h"
#include "affine.h"
#include "output.h"
#include "calc.h"
#include "tree.h"
#include "cmdline.h"


void print_usage(char *s)
{
  fprintf(stderr, "Usage: %s rsys wge\n", s);
  exit(1);
}

void output_eks(list *s1, list *s2, char *typ)
{
  int i;
  
  printf("KL_poly([");
  for (i = 0; i < l_length(s1); i++)
    printf(i == 0 ? "%d" : ",%d", (int) l_element(s1, i));
  
  printf("], [");
  for (i = 0; i < l_length(s2); i++)
    printf(i == 0 ? "%d" : ",%d", (int) l_element(s2, i));
  printf("], %s)\n", typ);
}      

void print_nodes(node *nd, affine *afn, char *typ, list *top)
{
  list *exp;
  int i;
  
  if ((n_color(nd) & 1) != 0)
    return;
  n_color(nd) |= 1;
  
  if (l_length(top) - n_length(nd) > 1)
    {
      exp = l_new();
      alcove_to_exp(exp, n_alcove(nd), afn);
#if 0
      printf("echo ");
      output_eks(exp, top, typ);
#endif
      output_eks(exp, top, typ);
      l_free(exp);
    }
  
  for (i = 0; i < n_nbelow(nd); i++)
    print_nodes(n_below(nd, i), afn, typ, top);
}

void clear_cols(node *nd)
{
  int i;
  
  if ((n_color(nd) & 1) == 0)
    return;
  n_color(nd) &= ~1;
  
  for (i = 0; i < n_nbelow(nd); i++)
    clear_cols(n_below(nd, i));
}

void recurse(node *nd, affine *afn, char *typ)
{
  int i;
  list *exp;
  
  if ((n_color(nd) & 2) != 0)
    return;
  n_color(nd) |= 2;
  
  exp = l_new();
  alcove_to_exp(exp, n_alcove(nd), afn);
  
  for (i = 0; i < n_nbelow(nd); i++)
    print_nodes(n_below(nd, i), afn, typ, exp);
  for (i = 0; i < n_nbelow(nd); i++)
    clear_cols(n_below(nd, i));
  for (i = 0; i < n_nbelow(nd); i++)
    recurse(n_below(nd, i), afn, typ);
  
  l_free(exp);
}

int main(int ac, char **av)
{
  list *s;
  treebase *tb;
  affine *afn;
  char typ[20];
  
  cmd_init(ac, av);
  afn = cmd_rsys();
  s = cmd_wge(afn);
  
  tb = build_tree(s, 0, afn);
  tb_reset_node_colors(tb);
  
  sprintf(typ, "%c%d", afn->root_system_type, afn_dim(afn));
  printf("maxobjects 100000\n");
  recurse(tb_node_list(tb, tb_max_length(tb)), tb_affine(tb), typ);
  
  exit(0);
}
