/*  Dynkin project programs
 *  Copyright (C) 1998 Anders S. Buch (abuch@mi.aau.dk)
 *  See the file LICENSE for license information.
 */

#include <stdio.h>

#include "list.h"
#include "alloc.h"
#include "matrix.h"
#include "file.h"
#include "output.h"
#include "rank.h"
#include "cmdline.h"


void print_usage(char *s)
{
  fprintf(stderr, "Usage: %s old.mat new.mat\n", s);
  exit(1);
}

int main(int ac, char **av)
{
  binfile *old, *new;
  char *old_nm, *new_nm;
  char s[255];
  int i, n, p, dim;
  int first_entry, entries_read, entries_total, percent;
  vector *v;
  list *buf;
  
  cmd_init(ac, av);
  old_nm = cmd_file(1);
  new_nm = cmd_file(1);
  
  if (strcmp(old_nm, new_nm) == 0)
    cmd_error("Files must be different.\n");
  
  if ((old = bf_openread(old_nm)) == NULL)
    cmd_error("Bad matrix file `%s'\n", old_nm);
  
  if ((new = bf_openwrite(new_nm, 8)) == NULL)
    cmd_error("Can't open `%s' for writing.\n", new_nm);
  
  get_string(old, s);
  put_string(new, s);
  n = atoi(s + 1);
  
  p = get_uint(old);
  put_uint(new, p);
  printf("Root system:  %s, p = %d\n", s, p);
  
  v = v_new(n);
  get_uvec(old, v);
  put_uvec(new, v);
  printf("lambda = ");
  for (i = 0; i < n; i++)
    putchar(fn_digit[v_entry(v, i)]);
  get_uvec(old, v);
  put_uvec(new, v);
  printf(" ;  mu = ");
  for (i = 0; i < n; i++)
    putchar(fn_digit[v_entry(v, i)]);
  putchar('\n');
  
  dim = get_uint(old);
  put_uint(new, dim);
  printf("Matrix dimension:  %d\n", dim);
  
  first_entry = get_uint(old);
  put_uint(new, first_entry);
  
  i = unsigned_bits(p);
  put_dflt_bits(new, i);
  
  entries_read = 0;
  buf = l_new();
  while (! bf_eof(old))
    {
      int x = get_uint(old);
      if (bf_isflushed(old))
	{
	  entries_read += l_length(buf);
	  for (i = 0; i < l_length(buf); i++)
	    put_uint(new, l_intelem(buf, i));
	  l_reset(buf);
	  bf_clearflush(old);
	}
      l_appendint(buf, x);
    }
  
  entries_total = dim * (dim + 1) / 2;
  percent = 1000 * entries_read / entries_total;
  printf("Entries in file:   %d  of  %d   (%d.%d%%)\n", entries_read, 
	 entries_total, percent / 10, percent % 10);
  
  l_free(buf);
  v_free(v);
  bf_close(old);
  bf_close(new);
  
  memory_report;
  
  exit(0);
}
