/*  Dynkin project programs
 *  Copyright (C) 1998 Anders S. Buch (abuch@mi.aau.dk)
 *  See the file LICENSE for license information.
 */

#include <stdio.h>

#include "alloc.h"
#include "matrix.h"
#include "file.h"
#include "output.h"
#include "rank.h"
#include "cmdline.h"


void print_usage(char *s)
{
  fprintf(stderr, "Usage: %s file.mat\n", s);
  exit(1);
}

int main(int ac, char **av)
{
  binfile *bf;
  char s[255];
  int n, p, dim, i, j;
  matrix *M;
  vector *v, *inv;
  int first_entry;
  char *name;
  
  cmd_init(ac, av);
  name = cmd_file(1);
  
  if ((bf = bf_openread(name)) == NULL)
    cmd_error("Bad matrix file `%s'\n", name);
  
  get_string(bf, s);
  puts(s);
  n = atoi(s + 1);
  p = get_uint(bf);
  inv = calc_inverse_list(p);
  
  v = v_new(n);
  get_uvec(bf, v);
  print_vec(v, 0);
  get_uvec(bf, v);
  print_vec(v, 0);
  
  dim = get_uint(bf);
  printf("dim = %d\n", dim);
  M = m_new(dim);
  
  first_entry = get_uint(bf);
  if (first_entry != 0)
    cmd_error("Matrix incomplete.\n");
  
  for (i = 0; i < dim; i++)
    {
      for (j = 0; j <= i; j++)
	{
	  int x = get_uint(bf);
	  m_entry(M, i, j) = x;
	  m_entry(M, j, i) = x;
	}
    }
  
  if (bf_eof(bf))
    cmd_error("Matrix incomplete.\n");
  
  printf("matrix read from file.\n");
  fflush(stdout);
  
  printf("rank: %d\n", rank_mod_p(M, p, inv));
  
  v_free(v);
  m_free(M);
  v_free(inv);
  bf_close(bf);
  
  memory_report;
  
  exit(0);
}
