#ifndef _MATRIX_H
#define _MATRIX_H

#include <string.h>
#include <stdlib.h>

#include "hashtab.h"

typedef struct _vector {
  int length;
  int e[1];
} vector;

typedef struct _matrix {
  int dim;
  int e[1];
} matrix;

#define v_length(v)		((v)->length)
#define v_entry(v, i)		((v)->e[i])
#define v_set_zero(v)		memset((v)->e, 0, (v)->length * sizeof(int))
#define v_set_unit(v, i)	{ v_set_zero(v); v_entry(v, i) = 1; }
#define v_copy(d, v)		\
	memcpy((d)->e, (v)->e, (d)->length * sizeof(int))

vector *v_new(int sz);
vector *v_new_zero(int sz);
vector *v_new_unit(int sz, int i);
vector *v_new_copy(vector *src);

#define v_free(v)		weyl_free(v)
void v_set_ones(vector *v);

void v_add(vector *dst, vector *v1, vector *v2);
void v_sub(vector *dst, vector *v1, vector *v2);
void v_scale(vector *dst, int r, vector *v);
void v_div(vector *dst, vector *v, int r);
void v_add_multiple(vector *dst, int r, vector *v);
int v_dot(vector *v1, vector *v2);
int v_sum_of_entries(vector *v);
int v_compare(vector *v1, vector *v2);
int v_lesseq(vector *v1, vector *v2);
hashkey_t v_hash(vector *v);

#define m_dim(m)		((m)->dim)
#define m_entry(m, i, j)	((m)->e[(i) * (m)->dim + (j)])
#define m_set_zero(m)		\
	memset((m)->e, 0, (m)->dim * (m)->dim * sizeof(int))
#define m_copy(d, m)		\
	memcpy((d)->e, (m)->e, (d)->dim * (d)->dim * sizeof(int))

matrix *m_new(int dim);
matrix *m_new_zero(int dim);
matrix *m_new_id(int dim);
void m_set_id(matrix *dst);
#define m_free(m)		weyl_free(m)

#define m_get_row(v, m, i) \
	memcpy((v)->e, (m)->e + (i) * (m)->dim, (v)->length * sizeof(int))
void m_get_column(vector *v, matrix *m, int j);

void m_add(matrix *dst, matrix *m1, matrix *m2);
void m_mult(matrix *dst, matrix *m1, matrix *m2);
void m_scale(matrix *dst, int r, matrix *m);
void m_div(matrix *dst, matrix *m, int r);
void m_transpose(matrix *dst, matrix *m);
int m_determinant(matrix *m);
void m_adjoint(matrix *dst, matrix *m, int *det);

void m_multvec(vector *dst, matrix *m, vector *v);
void m_vecmult(vector *dst, vector *v, matrix *m);

#endif
