/*  Dynkin project programs
 *  Copyright (C) 1998 Anders S. Buch (abuch@mi.aau.dk)
 *  See the file LICENSE for license information.
 */

#include <stdio.h>
#include <ctype.h>

#include "alloc.h"
#include "matrix.h"
#include "affine.h"
#include "output.h"
#include "sconst.h"
#include "cmdline.h"


void print_usage(char *s)
{
  fprintf(stderr, "Usage: %s rsys\n", s);
  exit(1);
}

int main(int ac, char **av)
{
  affine *afn;
  matrix *Npos, *Nneg, *add_mat, *sub_mat;
  list *plus_roots;
  
  cmd_init(ac, av);
  afn = cmd_rsys();
  
  plus_roots = afn->positive_roots;
  add_mat = build_add_matrix(plus_roots, 1);
  sub_mat = build_add_matrix(plus_roots, -1);
  
  Npos = m_new(l_length(plus_roots));
  Nneg = m_new(l_length(plus_roots));
  calc_sconsts(afn, add_mat, sub_mat, Npos, Nneg);
  
  printf("N(a,b):\n");
  print_table(plus_roots, Npos);
  printf("\nN(a,-b):\n");
  print_table(plus_roots, Nneg);
  
  afn_free(afn);
  m_free(add_mat);
  m_free(sub_mat);
  m_free(Npos);
  m_free(Nneg);
  
  memory_report;
  
  exit(0);
}
