/*  Dynkin project programs
 *  Copyright (C) 1998 Anders S. Buch (abuch@mi.aau.dk)
 *  See the file LICENSE for license information.
 */

#include "matrix.h"
#include "rank.h"

#ifdef __GNUC__
inline
#endif
int mod(int a, int p)
{
  a %= p;
  return (a < 0) ? a + p : a;
}

vector *calc_inverse_list(int p)
{
  vector *inv = v_new(p);
  int i;
  
  v_entry(inv, 0) = 0;
  for (i = 1; i < p; i++)
    {
      int x = i;
      int y = p;
      int x1 = 1;
      int y1 = 0;
      
      while (x != 0)
	{
	  int t = y / x;
	  int z = y - t * x;
	  int z1 = y1 - t * x1;
	  y = x;
	  y1 = x1;
	  x = z;
	  x1 = z1;
	}
      if (y < 0)
	y1 = - y1;
      v_entry(inv, i) = mod(y1, p);
    }
  return inv;
}

void reduce_mat_mod_p(matrix *m, int p)
{
  int n = m_dim(m);
  int i, j;
  
  for (i = 0; i < n; i++)
    for (j = 0; j < n; j++)
      m_entry(m, i, j) = mod(m_entry(m, i, j), p);
}

int rank_mod_p(matrix *m, int p, vector *inv)
{
  int r, c, i, j, x;
  int n = m_dim(m);
  
  r = 0;
  c = 0;
  while (c < n)
    {
      i = r;
      while (i < n && m_entry(m, i, c) == 0)
	i++;
      if (i < n)
	{
	  if (i != r)
	    {
	      for (j = c; j < n; j++)
		{
		  int tmp = m_entry(m, r, j);
		  m_entry(m, r, j) = m_entry(m, i, j);
		  m_entry(m, i, j) = tmp;
		}
	    }
	  x = v_entry(inv, m_entry(m, r, c));
	  for (i = r + 1; i < n; i++)
	    {
	      int t = x * m_entry(m, i, c);
	      for (j = c + 1; j < n; j++)
		m_entry(m, i, j) = 
		  mod(m_entry(m, i, j) - t * m_entry(m, r, j), p);
	    }
	  r++;
	}
      c++;
    }
  return r;
}
