/*  Dynkin project programs
 *  Copyright (C) 1998 Anders S. Buch (abuch@mi.aau.dk)
 *  See the file LICENSE for license information.
 */

#include <stdio.h>

#include "alloc.h"
#include "matrix.h"
#include "affine.h"
#include "output.h"
#include "sconst.h"
#include "efstring.h"
#include "rank.h"
#include "weight.h"
#include "freudlib.h"
#include "cmdline.h"


void print_usage(char *s)
{
  fprintf(stderr, "Usage: %s rsys-p weight\n", s);
  exit(1);
}

int main(int ac, char **av)
{
  affine *afn;
  int n, l;
  vector *a, *aw, *lambda;
  list *sums;
  weight_set *hw_set;
  list *omega;
  
  cmd_init(ac, av);
  afn = cmd_rsys();
  n = m_dim(afn->cartan);
  lambda = cmd_weight(0, afn);
  
  hw_set = calc_weights_below(lambda, afn);
  omega = ws_to_list(hw_set);
  
  a = v_new(n);
  aw = v_new(n);
  
  for (l = 0; l < l_length(omega); l++)
    {
      weight_set *ws = l_element(omega, l);
      vector *w = ws_weight(ws);
      
      v_sub(aw, lambda, w);
      m_multvec(a, afn->atcartan, aw);
      v_div(a, a, afn->dcartan);
      
      sums = generate_sums(a, afn);
      ws_mult(ws) = l_length(sums);
      free_sums(sums);
    }
  
  print_weight_set(hw_set, 1, 0);
  
  afn_free(afn);
  v_free(lambda);
  v_free(a);
  v_free(aw);
  ws_free(hw_set);
  l_free(omega);
  
  memory_report;
  
  exit(0);
}
