#ifndef _TREE_H
#define _TREE_H

#include "affine.h"
#include "list.h"
#include "matrix.h"
#include "hashtab.h"
#include "polynomial.h"

typedef struct _node {
  int length;
  vector *alcove;
  int number;
  struct _node *next;
  union {
    int color;
    unsigned *colbits;
    void *data;
  } u;
  int nbelow;
  struct _node *below[1];
} node;

#if 0
typedef struct _tree {
  node *node;
  struct _tree *next[1];
} tree;
#endif

typedef struct _treebase {
  int dim;
  affine *afn;
  hashtab *nodeset;
  
  int max_length;
  int node_count_sz;
  int *node_count;
  node **node_list;
  
  unsigned **bit_matrices;
  unsigned *all_colbits;
  hashtab *kl_cache;
  polynomial *zero;
  polynomial *one;
} treebase;


#define n_length(n)	((n)->length)
#define n_alcove(n)	((n)->alcove)
#define n_number(n)	((n)->number)
#define n_next(n)	((n)->next)
#define n_color(n)	((n)->u.color)
#define n_colbits(n)	((n)->u.colbits)
#define n_data(n)	((n)->u.data)
#define n_ptree(n)	((n)->ptree)
#define n_nbelow(n)	((n)->nbelow)
#define n_below(n,i)	((n)->below[i])

#define t_node(t)	((t)->node)
#define t_next(t,i)	((t)->next[i])

#define tb_dim(tb)		((tb)->dim)
#define tb_affine(tb)		((tb)->afn)
#define tb_nodeset(tb)		((tb)->nodeset)
#define tb_max_length(tb)	((tb)->max_length)
#define tb_node_count_sz(tb)	((tb)->node_count_sz)
#define tb_node_count(tb,l)	((tb)->node_count[l])
#define tb_row_length(tb,l)	((tb_node_count(tb,l) + WORD_BIT - 1)/WORD_BIT)
#define tb_node_list(tb,l)	((tb)->node_list[l])
#define tb_bit_matrix(tb,l1,l2)	\
	((tb)->bit_matrices[(l1) + ((tb)->max_length - 1) * ((l2) - 2) - 1])
#define tb_all_colbits(tb)	((tb)->all_colbits)

#define tb_kl_cache(tb)		((tb)->kl_cache)
#define tb_zero(tb)		((tb)->zero)
#define tb_one(tb)		((tb)->one)

#define tb_min_element(tb)	t_node(tb_tree(tb))

node *n_new(int length, vector *alc);
#define n_insbelow(n, b)	  ((n)->below[((n)->nbelow)++] = (b))
void n_free(node *nd);

#if 0
tree *t_new(int dim);
void t_free(tree *t, int dim);
tree *t_find(tree *t, list *exp);
tree *t_mkfind(tree *t, list *exp, int dim);
#endif

treebase *tb_new(affine *afn);
void tb_free(treebase *tb);
void tb_set_max_length(treebase *tb, int length);
node * tb_find(treebase *tb, list *exp);
#define tb_findalc(tb,a)	(hash_lookup(tb_nodeset(tb),(a)))
void tb_reset_node_colors(treebase *tb);
void tb_init_colbits(treebase *tb);

#endif
